import numpy as np
import torch

from labml.configs import BaseConfigs, option


class SetSeed:
    def __init__(self, seed: int):
        self.seed = seed

    def __call__(self):
        torch.manual_seed(self.seed)
        np.random.seed(self.seed)


class SeedConfigs(BaseConfigs):
    r"""
    This is a configurable module for setting the seeds.
    It will set seeds with ``torch.manual_seed`` and ``np.random.seed``.

    You need to call ``set`` method to set seeds
    (`example <https://github.com/labmlai/labml/blob/master/samples/pytorch/mnist/e_labml_helpers.py>`_).

    Arguments:
        seed (int): Seed integer. Defaults to ``5``.
    """
    seed: int = 5

    set = '_set_seed'


@option(SeedConfigs.set)
def _set_seed(c: SeedConfigs):
    return SetSeed(c.seed)



import dataclasses

import torch

from labml import tracker
from . import Metric


@dataclasses.dataclass
class RecallPrecisionState:
    fn: int = 0
    fp: int = 0
    tn: int = 0
    tp: int = 0

    def reset(self):
        self.fn = 0
        self.fp = 0
        self.tn = 0
        self.tp = 0

    @property
    def total(self):
        return self.fn + self.fp + self.tn + self.tp


class RecallPrecision(Metric):
    data: RecallPrecisionState

    def __call__(self, output: torch.Tensor, target: torch.Tensor):
        pred = output.view(-1) > 0
        target = target.view(-1)

        self.data.fn += ((pred == 0) & (target == 1)).sum().item()
        self.data.fp += ((pred == 1) & (target == 0)).sum().item()
        self.data.tn += ((pred == 0) & (target == 0)).sum().item()
        self.data.tp += ((pred == 1) & (target == 1)).sum().item()

    def create_state(self):
        return RecallPrecisionState()

    def set_state(self, data: any):
        self.data = data

    def on_epoch_start(self):
        self.data.reset()

    def on_epoch_end(self):
        self.track()

    def track(self):
        if self.data.tp + self.data.fp > 0:
            tracker.add("prcn.", self.data.tp / (self.data.tp + self.data.fp))
        if self.data.tp + self.data.fn > 0:
            tracker.add("recl.", self.data.tp / (self.data.tp + self.data.fn))
